package fishrungames.bashgid.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;


import fishrungames.bashgid.MainActivity;

public class NewsManager {

	private static NewsManager instance = null;
	
	//Xperimental -- need do something with this. Change to semaphore?
	public static volatile boolean SynchronizationInProcess = false;
	
	
	public static NewsManager GetInstance()
	{
		if (instance == null)
		{
			instance = new NewsManager();
		}
		
		return instance;
	}
	
	
	public static class NewsRecord
	{
		public String title;
		public String description;
		public String imageId;
		public Date pubDate;
		
		public NewsRecord(String title, String description, String imageId, Date pubDate)
		{
			this.title = title;
			this.description = description;
			this.imageId = imageId;
			this.pubDate = pubDate;
		}
		
		NewsRecord(RssReader.Rss2Item rss2Item)
		{
			this.title = rss2Item.title;
			this.description = rss2Item.description;
			
			if ((rss2Item.rss2Enclosure != null) && (DownloadManager.MimeTypeIsImage(rss2Item.rss2Enclosure.mimeType)))
			{
				this.imageId = rss2Item.rss2Enclosure.url;
			}
			else
			{
				this.imageId = "R.drawable.transparent";
			}
			
			this.pubDate = rss2Item.pubDate;
		}
	}
	
	ArrayList<NewsRecord> newsRecord = new ArrayList<NewsRecord>();
	
	public RemoveCallbackInterface DownloadNews(NewsUpdatedCallbackInterface callback)
	{
		
		DownloadNewsRunnable downloadNewsRunnable = new DownloadNewsRunnable(callback);
		
		Thread thread = new Thread(downloadNewsRunnable);
		
		thread.start();
		
		return downloadNewsRunnable;
	}
	
	class DownloadNewsRunnable implements Runnable, RemoveCallbackInterface
	{
		
		NewsUpdatedCallbackInterface callback;
		
		public DownloadNewsRunnable(NewsUpdatedCallbackInterface callback)
		{
			this.callback = callback;
		}
		
		@Override
		public void RemoveCallback()
		{
			callback = null;
		}
		
		@Override
		public void run()
		{
			if (SynchronizationInProcess)
			{
				return;
			}
			
			SynchronizationInProcess = true;
			
			final String [] urlArr = {
					"https://www.bashkortostan.ru/presscenter/news/rss/",
					"http://www.bashedu.ru/rss.xml",
					"http://www.bashinform.ru/rss/all.xml",
					"http://www.minkult-rb.ru/news/rss/",
					"http://02.mvd.ru/news/rss/"
					
					}; 
			
			ArrayList<NewsRecord> localNewsRecord = MainActivity.getInstance().newsDataSource.getNews();
			
			ArrayList<String> imageToDownloadList = new ArrayList<String>();
			
			for (int i = 0; i < urlArr.length; i++)
			{
			
				LoadNewsAndImagesFromRss2(urlArr[i], localNewsRecord, imageToDownloadList);
				
				Collections.sort(localNewsRecord, new NewsSortComparator());
				
				//NewsSortComparator
				
				//Xperimental -- need mutex right here
				newsRecord = localNewsRecord;
				
				MainActivity.getInstance().newsDataSource.replaceNews(newsRecord);
				
				//Xperimental -- need mutex here, too
				if (callback != null)
				{
					callback.OnNewsUpdated();
				}
			
			}
			
			for (String imageToDownload : imageToDownloadList)
			{
				ImageManager.getInstance().DownloadImageIfNeeded(imageToDownload);
			}
			
			SynchronizationInProcess = false;
		}
	}
	
	public static interface NewsUpdatedCallbackInterface
	{
		public void OnNewsUpdated();
	}
	
	public static interface RemoveCallbackInterface
	{
		public void RemoveCallback();
	}
	
	
	public NewsManager()
	{
		newsRecord = MainActivity.getInstance().newsDataSource.getNews();
		
	}
	
	
	public ArrayList<NewsRecord> getNews()
	{
		return newsRecord;
	}
	
	
	public static ArrayList<String> ExtractEnclosureImageUrl(ArrayList<RssReader.Rss2Item> rss2ItemArr)
	{
		ArrayList<String> imageToDownloadList = new ArrayList<String>();
		
		for (RssReader.Rss2Item rss2Item : rss2ItemArr)
		{
			//Enclosure
			if (rss2Item.rss2Enclosure != null)
			{
				if (DownloadManager.MimeTypeIsImage(rss2Item.rss2Enclosure.mimeType))
				{
					imageToDownloadList.add(rss2Item.rss2Enclosure.url);
				}
			}
		}
		
		return imageToDownloadList;
	}
	
	public static ArrayList<String> ExtractDescriptionImageUrl(ArrayList<RssReader.Rss2Item> rss2ItemArr)
	{
		ArrayList<String> imageToDownloadList = new ArrayList<String>();
		
		for (RssReader.Rss2Item rss2Item : rss2ItemArr)
		{
			//And images inside text
			int i = rss2Item.description.indexOf("<img");
			
			while (i != -1)
			{
				int j = rss2Item.description.indexOf("src=\"", i);
				
				if (j != -1)
				{
					j += ("src=\"").length();
					
					int k = rss2Item.description.indexOf("\"", j);
					
					String imageUrl = rss2Item.description.substring(j, k);
		
					imageToDownloadList.add(imageUrl);
				}
				
				i = rss2Item.description.indexOf("<img", j);
			}
		}
		
		return imageToDownloadList;
	}
	
	public static ArrayList<String> ConvertRelativeToAbsoluteImageUrlList(ArrayList<String> relativeImageUrlList, String url)
	{
		ArrayList<String> absoluteImageUrlList = new ArrayList<String>();
		
		for (String relativeImageUrl : relativeImageUrlList)
		{
			if (!relativeImageUrl.startsWith("http://"))
			{
				String homeFolder = DownloadManager.GetUrlFolder(url);
				
				relativeImageUrl = homeFolder + relativeImageUrl;
			}
			
			absoluteImageUrlList.add(relativeImageUrl);
		}
		
		return absoluteImageUrlList;
	}
	
	
	static void ReplaceRemoteUrlToLocalUrl(ArrayList<RssReader.Rss2Item> rss2ItemArr, ArrayList<String> relativeImageUrlList, ArrayList<String> absoluteImageUrlList)
	{

		for (RssReader.Rss2Item rss2Item : rss2ItemArr)
		{
			for (int i = 0; i < relativeImageUrlList.size(); i++)
			{
				String fileName = ImageManager.getInstance().GetImageLocalUrl(absoluteImageUrlList.get(i));
			
				rss2Item.description = rss2Item.description.replace("\"" +relativeImageUrlList.get(i) + "\"" , "\"" +fileName + "\"");
			}
		}
	}
	
	static void LoadNewsAndImagesFromRss2(String url, ArrayList<NewsRecord> newsRecordArr, ArrayList<String> imageUrlArr)
	{
		//Xperimental -- need to optimize this code. slow block BEGINS
		String xmlCode = DownloadManager.getXmlFromUrl(url);
		
		if (xmlCode == null)
		{
			return;
		}
		
		//Remove BOM character if present
		if (xmlCode.startsWith("\uFEFF"))
		{
			xmlCode = xmlCode.substring(1);
	    }
		
		
		
		RssReader rssReader = new RssReader();
		
		rssReader.ReadRss(xmlCode);
		
		
		//Xperimental -- need to optimize this code. slow block ENDS
		
		imageUrlArr.addAll(ExtractEnclosureImageUrl(rssReader.rss2ItemArr));
		
		
		ArrayList<String> relativeImageUrlList = ExtractDescriptionImageUrl(rssReader.rss2ItemArr);
		
		ArrayList<String> absoluteImageUrlList = ConvertRelativeToAbsoluteImageUrlList(relativeImageUrlList, url);
		
		imageUrlArr.addAll(absoluteImageUrlList);
		
		//Xperimental -- this function is very slow, need to speed up
		ReplaceRemoteUrlToLocalUrl(rssReader.rss2ItemArr, relativeImageUrlList, absoluteImageUrlList);			
					
		for (RssReader.Rss2Item rss2Item : rssReader.rss2ItemArr)
		{
			boolean exists = false;
			
			for (NewsRecord newsRecord : newsRecordArr)
			{
				if (newsRecord.title.equals(rss2Item.title))
				{
					exists = true;
					break;
				}
			}
			
			if (!exists)
			{
				newsRecordArr.add(new NewsRecord(rss2Item));
			}
		}		
			
	}
	
	public static class NewsSortComparator implements Comparator<NewsRecord> {
	    public int compare(NewsRecord newsRecord1, NewsRecord newsRecord2) {
	        return newsRecord2.pubDate.compareTo(newsRecord1.pubDate);
	    }
	}

	
}
