package fishrungames.bashgid.core.db;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Locale;

import fishrungames.bashgid.core.NewsManager;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;

public class NewsDataSource
{
	
	//Xperimental -- move somewhere
	public static final SimpleDateFormat iso8601Format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US);

	private SQLiteDatabase database;

	private BashgidSqliteHelper dbHelper;

	private String[] allColumns = { BashgidSqliteHelper.COLUMN_ID, BashgidSqliteHelper.COLUMN_TITLE, BashgidSqliteHelper.COLUMN_DESCRIPTION,
			BashgidSqliteHelper.COLUMN_IMAGE_ID, BashgidSqliteHelper.COLUMN_PUBDATE };

	public NewsDataSource(Context context) {
		dbHelper = new BashgidSqliteHelper(context);
	}

	public void open() throws SQLException
	{
		database = dbHelper.getWritableDatabase();
	}

	public void close()
	{
		dbHelper.close();
	}

	public void createNewsRecord(NewsManager.NewsRecord newsRecord)
	{

		ContentValues values = new ContentValues();

		values.put(BashgidSqliteHelper.COLUMN_TITLE, newsRecord.title);
		values.put(BashgidSqliteHelper.COLUMN_DESCRIPTION, newsRecord.description);
		values.put(BashgidSqliteHelper.COLUMN_IMAGE_ID, newsRecord.imageId);
		values.put(BashgidSqliteHelper.COLUMN_PUBDATE, iso8601Format.format(newsRecord.pubDate));
		
		database.insert(BashgidSqliteHelper.TABLE_NEWS, null, values);
		
	}

	public void replaceNews(ArrayList<NewsManager.NewsRecord> newsRecordArr)
	{

		database.delete(BashgidSqliteHelper.TABLE_NEWS, null, null);

		for (NewsManager.NewsRecord newsRecord : newsRecordArr)
		{
			createNewsRecord(newsRecord);
		}

	}

	public ArrayList<NewsManager.NewsRecord> getNews()
	{

		ArrayList<NewsManager.NewsRecord> newsRecordArr = new ArrayList<NewsManager.NewsRecord>();

		Cursor cursor = database.query(BashgidSqliteHelper.TABLE_NEWS, allColumns, null, null, null, null, null);

		cursor.moveToFirst();
		while (!cursor.isAfterLast())
		{
			NewsManager.NewsRecord newsRecord = cursorToNewsRecord(cursor);
			newsRecordArr.add(newsRecord);
			cursor.moveToNext();
		}
		// make sure to close the cursor
		cursor.close();
		return newsRecordArr;
	}

	private NewsManager.NewsRecord cursorToNewsRecord(Cursor cursor)
	{

		NewsManager.NewsRecord newsRecord = null;
		try
		{
			newsRecord = new NewsManager.NewsRecord(cursor.getString(1), cursor.getString(2), cursor.getString(3), iso8601Format.parse(cursor.getString(4)));
		}
		catch (ParseException e)
		{
			e.printStackTrace();
		}

		return newsRecord;
	}
}
